%---------------------------------------------------------------------------
% AtaxiaEvaluation is a software application property of University of Oviedo
%        Instrumentation and Measurement Lab - Campus de Viesques
%                          Version 1.1 2019
%---------------------------------------------------------------------------

function varargout = AtaxiaEvaluation(varargin)

    % Initial code.
    gui_Singleton = 1;
    gui_State = struct('gui_Name',       mfilename, ...
                       'gui_Singleton',  gui_Singleton, ...
                       'gui_OpeningFcn', @AtaxiaEvaluation_OpeningFcn, ...
                       'gui_OutputFcn',  @AtaxiaEvaluation_OutputFcn, ...
                       'gui_LayoutFcn',  [] , ...
                       'gui_Callback',   []);
    if nargin && ischar(varargin{1})
        gui_State.gui_Callback = str2func(varargin{1});
    end

    if nargout
        [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
    else
        gui_mainfcn(gui_State, varargin{:});
    end
    % End of the initial code.

% Initial conditions when executing the program.
function AtaxiaEvaluation_OpeningFcn(hObject, eventdata, handles, varargin)
    % Pointer to access the property values of the elements and the values
    % of the program variables.
    handles.output = hObject;
    % Save the application data.
    guidata(hObject, handles);

% Function that returns values to the main program.
function varargout = AtaxiaEvaluation_OutputFcn(hObject, eventdata, handles) 
    varargout{1} = handles.output;

%Connect button initiates the connection to the hardware device.
function Connect_Callback(hObject, eventdata, handles)
    global dataTot databuf 
    % The variables are initialized.
    bufferSize=2000;
    dataTot=[];
    databuf = zeros(bufferSize,4);
    mode=1;
    
    % Create and open the serial port to connect to the hardware device.
    serialPort = crsOpenSerialPort('BlueGain');
    % Send a reset command to the hardware device to clear the buffer.
    BlueGainResetSampling(serialPort, mode);
    % Send a data request initially to the hardware device.
    BlueGainRequestData(serialPort, mode);
    % The data generated by the hardware device is collected and the continuous 
    % EOG signal is drawn.
    while 1
        % Collect data from the serial port.
        dataPacket = BlueGainRetrieveData(serialPort, mode);
            
        %A data request is sent to the hardware device to fill the buffer.
        if ~isempty(dataPacket)
            BlueGainRequestData(serialPort, mode);
        end

        % If the application is open, the buffer is filled.
        pause(0.05);
            
        % Update databuf if dataPacket is not empty.
        if ~isempty(dataPacket)
           databuf = [databuf(size(dataPacket,1)+1:end,:);dataPacket(:,[1 2 3 4])];
           dataTot=[dataTot; dataPacket(:,[1 2 3 4]) ];
           save('Data.txt','dataTot','-ascii','-tabs');
        end
    end
    
% This function is executed when Exit button is pressed.
function Exit_Callback(hObject, eventdata, handles)
    %A new window appears, asking if you want to leave. 
    %If the answer is affirmative, the program is closed. 
    %If the answer is negative it returns to the program.
    answer=questdlg('Do you want to quit?','EXIT','Yes','No','No');
    if strcmp(answer,'No')
        return;
    else
        close all
    end

% This function is executed when Start button is pressed.
function Start_Callback(hObject, eventdata, handles)
    global dataTot databuf
    % The variables are initialized.
    dataTot=[];
    cleanV=[];
    % Clears the results graph each time the test is executed.
    cla(handles.axes1)
    set(handles.text1,'string',cleanV);
    % Open the video of the test in the video player.
    winopen('stimulus.avi')

% Pressing Save button saves the signal obtained from the execution of
% the test.
function Save_Callback(hObject, eventdata, handles)
    global dataTot databuf
    % The signal is saved in .txt format.
    save('Signal.txt','dataTot','-ascii','-tabs');

% Results button executes this function that processes the signal stored
% by pressing Save button.
function Evaluate_Callback(hObject, eventdata, handles)
    % The variables are initialized.
    derived=[];
    time=[];

    % The values of the signal recorded during the test are recovered.
    funcion=load('Signal.txt');
    Channel1=funcion(:,3);  % Horizontal channel
    %Channel2=funcion(:,4); % Vertical channel
    temp=0:1:length(Channel1)-1;
    
    %Low pass, ninth order, Butterworth filter.
    [z,p,k]=buttap(9);
    [num,den]=zp2tf(z,p,k);
    [h,f]= freqz(num,den,length(temp));
    A=abs(h);
    
    % The Fourier Transform of the signal generated in the test is 
    % calculated so that it can be multiplied by the filter.
    TF=fft(Channel1,length(temp));
    Y=TF.*A;
    
    %The inverse Fourier Transform is calculated.
    Signal=ifft(Y,length(temp));
    
    %Real signal in the time domain.
    Signal=real(Signal);

    %First filter of mobile media.
    nf=100;
    Signal=filter(ones(1,nf)/nf,1,Signal);
    
    % --------------------- Diagnostic test parameters --------------------
    % 1) Involuntary saccades in smooth tracking
    % ---------------------------------------------------------------------
    % The derivative of the signal is calculated by means of the equation 
    % of finite differences centered.
    for i = 1:(length(temp)-2)
        a=Signal(i);
        b=Signal(i+2);
        %Derived in i+1
        f=(b-a)/2;
        derived=[derived; f]; 
        time=[time,temp(i+1)];
        i=i+1;
    end

    %Second filter of mobile media.
    derived=filter(ones(1,nf)/nf,1,derived);
    
    %The processed signal is drawn. 
        plot(handles.axes1,(time-time(1))/1000,derived,'k')
        ylim(handles.axes1,[-10 10]);  
        xlim(handles.axes1,[0 length(time)/1000]); %In seconds
        set (handles.axes1, 'YColor', 'k', 'XColor', 'k','FontSize', 11, 'FontWeight','bold')
        ylabel(handles.axes1,'Voltage (mV)','FontSize', 11); 
        xlabel(handles.axes1,'Time (s)','FontSize', 11, 'FontWeight','bold'); 
        hold on
        
        %'Color',[222/255 235/255 250/255]

    % The peaks that are above the threshold (4mV) are searched. 
    derivedAbs=abs(derived);
    [peak_value, peak_location] = findpeaks(derivedAbs,'MINPEAKHEIGHT',3);
          
    % the frequency with which the saccades are produced is calculated, 
    % taking into account the duration of the test.
    rateOfSaccades=length(peak_location)/(length(time)/1000);
    rateOfSaccades=round(rateOfSaccades,2);
    
    set(handles.text4,'String',rateOfSaccades);
    
    % ---------------------------------------------------------------------
    % 2) Delay in the response to the stimulus movement
    % ---------------------------------------------------------------------  
    % The variables are initialized.    
    % samplingTime=0.001; % 1000 samples per second from hardware device.
    startPeriodL=5000;  % 5 s.
    sizeL=60000;
    delayPeriod=zeros(sizeL, 1); % A block of memory is allocated.
    
    % Latency is calculated from the start of the movement of the stimulus
    % at 5 s.
    for j = startPeriodL:(length(temp)-2)
        delayPeriod(j-startPeriodL+1)=Signal(j);
    end
    
    latencyPeriodAbs=abs(delayPeriod);
    
    % Calculation of how many samples pass until the first voltage increase
    % is found.
    [peak_value1, delay] = findpeaks(latencyPeriodAbs,'NPEAKS', 1);
    
    %The start of stimulus movement is drawn.
    plot(handles.axes1,[5 5],[-15 15], 'k')
    
    delayPlot=delay+startPeriodL;
    
    %The saccadic delay is drawn.
    plot(handles.axes1,[delayPlot/1000 delayPlot/1000],[-15 15], 'k--')
    hold off
    
    set(handles.text6,'String',delay);
    
    % ---------------------------------------------------------------------
    % 3) Deviation
    % ---------------------------------------------------------------------
    % The variables are initialized.
    % The sensitivity of the horizontal eye movements is 16V/
    sensitivity=16*10^-3; % In mV
    sizeD=3000;
    
    % During the first 2 s. the gaze is focused on the stimulus and the
    % deviation in the fixation is recorded during the following 3 s.
    startPeriodD=2000;  
    endPeriodD=5000; 
    fixation=zeros(sizeD, 1); % A block of memory is allocated.
    
    % The deviation that occurs during 3 s that the stimulus is in the 
    % center is calculated.
    for l = startPeriodD:endPeriodD
        fixation(l-startPeriodD+1)=Signal(l);
    end
    
    % The voltage values are converted to degrees.
    grades=fixation.*sensitivity;
    gradesAbs=abs(grades);
    
    % The maximum deviation in this period is obtained.
    maxDeviation=max(gradesAbs);
    maxDeviation=round(maxDeviation,2);
    set(handles.text8,'String',maxDeviation);
    
    % ---------------------------------------------------------------------
    % The evaluation based on the three parameters is shown.
    % The posible conditions that must be met to diagnose ataxia are:
    % 1) There are involuntary saccades in the smooth pursuit of the stimulus.
    % 2) There is an abnormal delay in the response to a stimulus (>300 ms.).
    % 3) There is an appreciable deviation in the fixation period (>3).
    
    if (rateOfSaccades>0.13) || (delay>300) || (maxDeviation>3)
        evaluation='POSITIVE';
    else
        evaluation='NEGATIVE';
    end
    
    set(handles.text1,'String',evaluation);
    
